@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
 
  *  Authors: Török Edvin, Kevin Lin
@@ -153,6 +153,7 @@ enum FunctionalityLevels {
     FUNC_LEVEL_0103_5    = 126, /**< LibClamAV release 0.103.5 */
     FUNC_LEVEL_0103_6    = 127, /**< LibClamAV release 0.103.6 */
     FUNC_LEVEL_0103_7    = 128, /**< LibClamAV release 0.103.7 */
+    FUNC_LEVEL_0103_8    = 129, /**< LibClamAV release 0.103.8 */
 
     FUNC_LEVEL_0104   = 140, /**< LibClamAV release 0.104.0 */
     FUNC_LEVEL_0104_1 = 141, /**< LibClamAV release 0.104.1 */
@@ -162,11 +163,26 @@ enum FunctionalityLevels {
 
     FUNC_LEVEL_0105   = 150, /**< LibClamAV release 0.105.0 */
     FUNC_LEVEL_0105_1 = 151, /**< LibClamAV release 0.105.1 */
+    FUNC_LEVEL_0105_2 = 152, /**< LibClamAV release 0.105.2 */
 
     FUNC_LEVEL_1_0   = 160, /**< LibClamAV release 1.0.0 */
     FUNC_LEVEL_1_0_1 = 161, /**< LibClamAV release 1.0.1 */
     FUNC_LEVEL_1_0_2 = 162, /**< LibClamAV release 1.0.2 */
     FUNC_LEVEL_1_0_3 = 163, /**< LibClamAV release 1.0.3 */
+    FUNC_LEVEL_1_0_4 = 164, /**< LibClamAV release 1.0.4 */
+    FUNC_LEVEL_1_0_5 = 165, /**< LibClamAV release 1.0.5 */
+
+    FUNC_LEVEL_1_1   = 180, /**< LibClamAV release 1.1.0 */
+    FUNC_LEVEL_1_1_1 = 181, /**< LibClamAV release 1.1.1 */
+    FUNC_LEVEL_1_1_2 = 182, /**< LibClamAV release 1.1.2 */
+    FUNC_LEVEL_1_1_3 = 183, /**< LibClamAV release 1.1.3 */
+
+    FUNC_LEVEL_1_2   = 190, /**< LibClamAV release 1.2.0 */
+    FUNC_LEVEL_1_2_1 = 191, /**< LibClamAV release 1.2.1 */
+    FUNC_LEVEL_1_2_2 = 192, /**< LibClamAV release 1.2.2 */
+
+    FUNC_LEVEL_1_3   = 200, /**< LibClamAV release 1.3.0 */
+    FUNC_LEVEL_1_3_1 = 201, /**< LibClamAV release 1.3.1 */
 };
 
 /**
