@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2019-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  EGG is an archive format created by ESTsoft used by their ALZip
  *  archiving software.
@@ -99,7 +99,7 @@ typedef uint16_t WCHAR;
  * general defines
  */
 #define EOFARC 0x08E28222 /* Signals end of each header, or end of archive. */
-//#define EOFAR_ 0x2282E208
+// #define EOFAR_ 0x2282E208
 /*
  * egg_header
  */
@@ -453,7 +453,7 @@ static cl_error_t egg_parse_encrypt_header(const uint8_t* index, size_t size, eg
      * The EGG specification (last updated 2016) for the encrypt header is not accurate.
      * The following describes my findings of the actual format for the encrypt header.
      *
-     * The significant discrepancy is that the Size includes the size of the header iself, not just the data following it.
+     * The significant discrepancy is that the Size includes the size of the header itself, not just the data following it.
      * No other extra_field header's size field includes the size of itself.
      * This must be accounted for by the caller of this function (see the "Fudge factor" comments where this function is used).
      *
@@ -475,7 +475,7 @@ static cl_error_t egg_parse_encrypt_header(const uint8_t* index, size_t size, eg
      *     |               |         | 6 | LEA256                                                                                                 |
      *     |---------------|---------|---|--------------------------------------------------------------------------------------------------------|
      *
-     * Depending on the Method (XOR / AES/LEA128 / AES/LEA256) The above will be be followed one of the following Method Headers:
+     * Depending on the Method (XOR / AES/LEA128 / AES/LEA256) The above will be followed by one of the following Method Headers:
      *
      *   XOR (KeyBase):
      *     |---------------|---------|------------------------------------------------------------------------------------------------------------|
@@ -579,11 +579,12 @@ static cl_error_t egg_parse_encrypt_header(const uint8_t* index, size_t size, eg
     }
 
     *encryptInfo = encrypt;
-    status       = CL_SUCCESS;
+    encrypt      = NULL;
+
+    status = CL_SUCCESS;
 
 done:
-
-    if (CL_SUCCESS != status) {
+    if (NULL != encrypt) {
         egg_free_encrypt(encrypt);
     }
 
@@ -608,8 +609,7 @@ static cl_error_t egg_parse_comment_header(const uint8_t* index, size_t size, ex
         /*
          * comment is encrypted, nothing to be done.
          */
-        *commentInfo = cli_strdup("<encrypted>");
-        status       = CL_EUNPACK;
+        status = CL_EUNPACK;
         goto done;
     }
 
@@ -623,7 +623,8 @@ static cl_error_t egg_parse_comment_header(const uint8_t* index, size_t size, ex
          */
         if (CL_SUCCESS != cli_codepage_to_utf8((char*)index, size, CODEPAGE_UTF8, &comment_utf8, &comment_utf8_size)) {
             cli_dbgmsg("egg_parse_comment_header: failed to convert codepage \"0\" to UTF-8\n");
-            comment_utf8 = cli_genfname(NULL);
+            status = CL_EUNPACK;
+            goto done;
         }
     } else {
         /* Should already be UTF-8. Use as-is.. */
@@ -637,9 +638,15 @@ static cl_error_t egg_parse_comment_header(const uint8_t* index, size_t size, ex
     cli_dbgmsg("egg_parse_comment_header: comment:          %s\n", comment_utf8);
 
     *commentInfo = comment_utf8;
-    status       = CL_SUCCESS;
+    comment_utf8 = NULL;
+
+    status = CL_SUCCESS;
 
 done:
+    if (NULL != comment_utf8) {
+        free(comment_utf8);
+    }
+
     return status;
 }
 
@@ -739,14 +746,14 @@ static cl_error_t egg_parse_block_headers(egg_handle* handle, egg_block** block)
     eggBlock->compressedData = (char*)index;
     handle->offset += blockHeader->compress_size;
 
-    *block = eggBlock;
+    *block   = eggBlock;
+    eggBlock = NULL;
+
     status = CL_SUCCESS;
 
 done:
-    if (CL_SUCCESS != status) {
-        if (eggBlock) {
-            egg_free_egg_block(eggBlock);
-        }
+    if (NULL != eggBlock) {
+        egg_free_egg_block(eggBlock);
     }
 
     return status;
@@ -963,8 +970,6 @@ static void print_posix_info_mode(uint32_t mode)
         printf("p");
     } else if (mode & POSIX_INFO_MODE_SYM_LINK) {
         printf("l");
-    } else if (mode & POSIX_INFO_MODE_SOCKET) {
-        printf("s");
     }
     /* Owner/Group/Other permissions */
     if (mode & POSIX_INFO_MODE_PERM_OWNER_READ) {
@@ -1207,34 +1212,21 @@ static cl_error_t egg_parse_file_extra_field(egg_handle* handle, egg_file* eggFi
 
             if (CL_SUCCESS != (retval = egg_parse_comment_header(index, size, extraField, &comment))) {
                 cli_dbgmsg("egg_parse_file_extra_field: Issue parsing comment header. Error code: %u\n", retval);
-                break;
+                // Don't fail out with a `goto done;`. We're making a best effort to process the file, so just move on.
+            } else if (comment == NULL) {
+                cli_errmsg("egg_parse_file_extra_field: Logic error! Successfully parsed comment header,"
+                           " but did not return egg_comment information!\n");
+                goto done;
             } else {
                 /*
-                 * Success?
+                 * Comment found. Add comment to our list.
                  */
-                if (comment == NULL) {
-                    /* Uh... no. */
-                    cli_errmsg("egg_parse_file_extra_field: Logic error! Succesfully parsed comment header,"
-                               " but did not return egg_comment information!\n");
-                    goto done;
-                } else {
-                    /*
-                     * Comment found. Add comment to our list.
-                     */
-                    char** comments_tmp;
-
-                    comments_tmp = (char**)cli_realloc(
-                        (void*)eggFile->comments,
-                        sizeof(char*) * (eggFile->nComments + 1));
-                    if (NULL == comments_tmp) {
-                        free(comment);
-                        status = CL_EMEM;
-                        goto done;
-                    }
-                    eggFile->comments                     = comments_tmp;
-                    eggFile->comments[eggFile->nComments] = comment;
-                    eggFile->nComments++;
-                }
+                CLI_REALLOC(eggFile->comments,
+                            sizeof(char*) * (eggFile->nComments + 1),
+                            free(comment),
+                            status = CL_EMEM);
+                eggFile->comments[eggFile->nComments] = comment;
+                eggFile->nComments++;
             }
             break;
         }
@@ -1446,14 +1438,14 @@ static cl_error_t egg_parse_file_headers(egg_handle* handle, egg_file** file)
         }
     }
 
-    *file  = eggFile;
+    *file   = eggFile;
+    eggFile = NULL;
+
     status = CL_SUCCESS;
 
 done:
-    if (CL_SUCCESS != status) {
-        if (eggFile) {
-            egg_free_egg_file(eggFile);
-        }
+    if (NULL != eggFile) {
+        egg_free_egg_file(eggFile);
     }
 
     return status;
@@ -1607,6 +1599,10 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
         return CL_EARG;
     }
 
+    *hArchive  = NULL;
+    *comments  = NULL;
+    *nComments = 0;
+
     handle = (egg_handle*)cli_calloc(1, sizeof(egg_handle));
     if (NULL == handle) {
         cli_errmsg("cli_egg_open: Failed to allocate memory for egg_handle.\n");
@@ -1666,22 +1662,15 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
                 cli_dbgmsg("cli_egg_open: Issue parsing file header. Error code: %u\n", retval);
                 goto done;
             } else if (found_file == NULL) {
-                cli_errmsg("cli_egg_open: Logic error! Succesfully parsed file headers,"
+                cli_errmsg("cli_egg_open: Logic error! Successfully parsed file headers,"
                            " but did not return egg_file information!\n");
                 goto done;
             } else {
                 /* Add file to list. */
-                egg_file** files_tmp;
-
-                files_tmp = (egg_file**)cli_realloc(
-                    (void*)handle->files,
-                    sizeof(egg_file*) * (handle->nFiles + 1));
-                if (NULL == files_tmp) {
-                    egg_free_egg_file(found_file);
-                    status = CL_EMEM;
-                    goto done;
-                }
-                handle->files                 = files_tmp;
+                CLI_REALLOC(handle->files,
+                            sizeof(egg_file*) * (handle->nFiles + 1),
+                            egg_free_egg_file(found_file),
+                            status = CL_EMEM);
                 handle->files[handle->nFiles] = found_file;
                 handle->nFiles++;
             }
@@ -1694,23 +1683,16 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
                 cli_dbgmsg("cli_egg_open: Issue parsing block header. Error code: %u\n", retval);
                 goto done;
             } else if (found_block == NULL) {
-                cli_errmsg("cli_egg_open: Logic error! Succesfully parsed block headers,"
+                cli_errmsg("cli_egg_open: Logic error! Successfully parsed block headers,"
                            " but did not return egg_block information!\n");
                 goto done;
             } else {
                 /* Add block to list. */
                 if (handle->bSolid) {
-                    egg_block** blocks_tmp;
-
-                    blocks_tmp = (egg_block**)cli_realloc(
-                        (void*)handle->blocks,
-                        sizeof(egg_block*) * (handle->nBlocks + 1));
-                    if (NULL == blocks_tmp) {
-                        egg_free_egg_block(found_block);
-                        status = CL_EMEM;
-                        goto done;
-                    }
-                    handle->blocks                  = blocks_tmp;
+                    CLI_REALLOC(handle->blocks,
+                                sizeof(egg_block*) * (handle->nBlocks + 1),
+                                egg_free_egg_block(found_block),
+                                status = CL_EMEM);
                     handle->blocks[handle->nBlocks] = found_block;
                     handle->nBlocks++;
                 } else {
@@ -1720,22 +1702,15 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
                      */
                     if (handle->nFiles == 0) {
                         cli_dbgmsg("cli_egg_open: No file found for block in non-solid archive.\n");
-                        // TODO: create an unamed block.
+                        // TODO: create an unnamed block.
+                        egg_free_egg_block(found_block);
                     } else {
-                        egg_block** blocks_tmp;
-
                         eggFile = handle->files[handle->nFiles - 1];
 
-                        /* Add block to list. */
-                        blocks_tmp = (egg_block**)cli_realloc(
-                            (void*)eggFile->blocks,
-                            sizeof(egg_block*) * (eggFile->nBlocks + 1));
-                        if (NULL == blocks_tmp) {
-                            egg_free_egg_block(found_block);
-                            status = CL_EMEM;
-                            goto done;
-                        }
-                        eggFile->blocks                   = blocks_tmp;
+                        CLI_REALLOC(eggFile->blocks,
+                                    sizeof(egg_block*) * (eggFile->nBlocks + 1),
+                                    egg_free_egg_block(found_block),
+                                    status = CL_EMEM);
                         eggFile->blocks[eggFile->nBlocks] = found_block;
                         eggFile->nBlocks++;
                     }
@@ -1745,7 +1720,6 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
             /*
              * Parse extra field for archive comment header.
              */
-            char** comments_tmp;
             extra_field* extraField = NULL;
             char* comment           = NULL;
             uint32_t size           = 0;
@@ -1795,24 +1769,32 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
                 goto done;
             }
 
-            retval = egg_parse_comment_header(index, size, extraField, &comment);
-            if (CL_SUCCESS != retval) {
-                cli_dbgmsg("cli_egg_open: Failed to parse archive comment extra_field data.\n");
-                goto done;
+            if (CL_SUCCESS != (retval = egg_parse_comment_header(index, size, extraField, &comment))) {
+                cli_dbgmsg("cli_egg_open: Issue parsing comment header. Error code: %u\n", retval);
+                // Don't fail out with a `goto done;`. We're making a best effort to process the file, so just move on.
+            } else {
+                /*
+                 * Success?
+                 */
+                if (comment == NULL) {
+                    /* Uh... no. */
+                    cli_errmsg("cli_egg_open: Logic error! Successfully parsed comment header,"
+                               " but did not return egg_comment information!\n");
+                    goto done;
+                } else {
+                    /*
+                     * Comment found. Add comment to our list.
+                     */
+                    CLI_REALLOC(handle->comments,
+                                sizeof(char*) * (handle->nComments + 1),
+                                free(comment),
+                                status = CL_EMEM);
+                    handle->comments[handle->nComments] = comment;
+                    handle->nComments++;
+                }
             }
-
-            comments_tmp = (char**)cli_realloc(
-                (void*)handle->comments,
-                sizeof(char*) * (handle->nComments + 1));
-            if (NULL == comments_tmp) {
-                free(comment);
-                status = CL_EMEM;
-                goto done;
-            }
-            handle->comments                    = comments_tmp;
-            handle->comments[handle->nComments] = comment;
-            handle->nComments++;
             handle->offset += size;
+
         } else {
             cli_dbgmsg("cli_egg_open: unexpected header magic:               %08x (%s)\n", magic, getMagicHeaderName(magic));
             status = CL_EPARSE;
@@ -1833,18 +1815,19 @@ cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t
         }
     }
 
-    *hArchive  = handle;
-    *comments  = handle->comments;
+    *comments        = handle->comments;
+    handle->comments = NULL;
+
     *nComments = handle->nComments;
 
+    *hArchive = handle;
+    handle    = NULL;
+
     status = CL_SUCCESS;
 
 done:
-    if (CL_SUCCESS != status) {
-        if (handle) {
-            egg_free_egg_handle(handle);
-        }
-        *hArchive = NULL;
+    if (NULL != handle) {
+        egg_free_egg_handle(handle);
     }
     return status;
 }
@@ -1937,7 +1920,6 @@ cl_error_t cli_egg_deflate_decompress(char* compressed, size_t compressed_size,
 {
     cl_error_t status = CL_EPARSE;
 
-    uint8_t* decoded_tmp;
     uint8_t* decoded = NULL;
     uint32_t declen = 0, capacity = 0;
 
@@ -1990,12 +1972,10 @@ cl_error_t cli_egg_deflate_decompress(char* compressed, size_t compressed_size,
     while (zstat == Z_OK && stream.avail_in) {
         /* extend output capacity if needed,*/
         if (stream.avail_out == 0) {
-            if (!(decoded_tmp = cli_realloc(decoded, capacity + BUFSIZ))) {
-                cli_errmsg("cli_egg_deflate_decompress: cannot reallocate memory for decompressed output\n");
-                status = CL_EMEM;
-                goto done;
-            }
-            decoded          = decoded_tmp;
+            CLI_REALLOC(decoded,
+                        capacity + BUFSIZ,
+                        cli_errmsg("cli_egg_deflate_decompress: cannot reallocate memory for decompressed output\n"),
+                        status = CL_EMEM);
             stream.next_out  = decoded + capacity;
             stream.avail_out = BUFSIZ;
             declen += BUFSIZ;
@@ -2040,7 +2020,9 @@ cl_error_t cli_egg_deflate_decompress(char* compressed, size_t compressed_size,
             break;
     }
 
-    *decompressed      = (char*)decoded;
+    *decompressed = (char*)decoded;
+    decoded       = NULL;
+
     *decompressed_size = declen;
 
     status = CL_SUCCESS;
@@ -2051,7 +2033,7 @@ done:
         (void)inflateEnd(&stream);
     }
 
-    if (CL_SUCCESS != status) {
+    if (NULL != decoded) {
         free(decoded);
     }
 
@@ -2063,7 +2045,6 @@ cl_error_t cli_egg_bzip2_decompress(char* compressed, size_t compressed_size, ch
 {
     cl_error_t status = CL_EPARSE;
 
-    char* decoded_tmp;
     char* decoded   = NULL;
     uint32_t declen = 0, capacity = 0;
 
@@ -2113,12 +2094,10 @@ cl_error_t cli_egg_bzip2_decompress(char* compressed, size_t compressed_size, ch
     while (bzstat == BZ_OK && stream.avail_in) {
         /* extend output capacity if needed,*/
         if (stream.avail_out == 0) {
-            if (!(decoded_tmp = cli_realloc(decoded, capacity + BUFSIZ))) {
-                cli_errmsg("cli_egg_bzip2_decompress: cannot reallocate memory for decompressed output\n");
-                status = CL_EMEM;
-                goto done;
-            }
-            decoded          = decoded_tmp;
+            CLI_REALLOC(decoded,
+                        capacity + BUFSIZ,
+                        cli_errmsg("cli_egg_bzip2_decompress: cannot reallocate memory for decompressed output\n");
+                        status = CL_EMEM);
             stream.next_out  = decoded + capacity;
             stream.avail_out = BUFSIZ;
             declen += BUFSIZ;
@@ -2157,7 +2136,9 @@ cl_error_t cli_egg_bzip2_decompress(char* compressed, size_t compressed_size, ch
             break;
     }
 
-    *decompressed      = (char*)decoded;
+    *decompressed = (char*)decoded;
+    decoded       = NULL;
+
     *decompressed_size = declen;
 
     status = CL_SUCCESS;
@@ -2166,7 +2147,7 @@ done:
 
     (void)BZ2_bzDecompressEnd(&stream);
 
-    if (CL_SUCCESS != status) {
+    if (NULL != decoded) {
         free(decoded);
     }
 
@@ -2178,7 +2159,6 @@ cl_error_t cli_egg_lzma_decompress(char* compressed, size_t compressed_size, cha
 {
     cl_error_t status = CL_EPARSE;
 
-    uint8_t* decoded_tmp;
     uint8_t* decoded = NULL;
     uint32_t declen = 0, capacity = 0;
 
@@ -2231,12 +2211,10 @@ cl_error_t cli_egg_lzma_decompress(char* compressed, size_t compressed_size, cha
     while (lzmastat == LZMA_RESULT_OK && stream.avail_in) {
         /* extend output capacity if needed,*/
         if (stream.avail_out == 0) {
-            if (!(decoded_tmp = cli_realloc(decoded, capacity + BUFSIZ))) {
-                cli_errmsg("cli_egg_lzma_decompress: cannot reallocate memory for decompressed output\n");
-                status = CL_EMEM;
-                goto done;
-            }
-            decoded          = decoded_tmp;
+            CLI_REALLOC(decoded,
+                        capacity + BUFSIZ,
+                        cli_errmsg("cli_egg_lzma_decompress: cannot reallocate memory for decompressed output\n");
+                        status = CL_EMEM);
             stream.next_out  = decoded + capacity;
             stream.avail_out = BUFSIZ;
             declen += BUFSIZ;
@@ -2274,7 +2252,9 @@ cl_error_t cli_egg_lzma_decompress(char* compressed, size_t compressed_size, cha
             break;
     }
 
-    *decompressed      = (char*)decoded;
+    *decompressed = (char*)decoded;
+    decoded       = NULL;
+
     *decompressed_size = declen;
 
     status = CL_SUCCESS;
@@ -2285,7 +2265,7 @@ done:
         (void)cli_LzmaShutdown(&stream);
     }
 
-    if (CL_SUCCESS != status) {
+    if (NULL != decoded) {
         free(decoded);
     }
 
@@ -2307,6 +2287,7 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
         goto done;
     }
 
+    *filename             = NULL;
     *output_buffer        = NULL;
     *output_buffer_length = 0;
 
@@ -2360,7 +2341,6 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
         }
 
         for (i = 0; i < currFile->nBlocks; i++) {
-            char* decompressed_tmp;
             egg_block* currBlock = currFile->blocks[i];
             cl_error_t retval    = CL_EPARSE;
 
@@ -2380,14 +2360,12 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
                         cli_warnmsg("cli_egg_extract_file: blockHeader compress_size != uncompress_size!\n");
                         break;
                     }
-                    decompressed_tmp = cli_realloc(decompressed, (size_t)decompressed_size + currBlock->blockHeader->compress_size);
-                    if (NULL == decompressed_tmp) {
-                        cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
-                                   decompressed_size);
-                        status = CL_EMEM;
-                        goto done;
-                    }
-                    decompressed = decompressed_tmp;
+
+                    CLI_REALLOC(decompressed,
+                                (size_t)decompressed_size + currBlock->blockHeader->compress_size,
+                                cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
+                                           decompressed_size),
+                                status = CL_EMEM);
 
                     memcpy(decompressed + decompressed_size, currBlock->compressedData, currBlock->blockHeader->compress_size);
                     decompressed_size += currBlock->blockHeader->compress_size;
@@ -2408,15 +2386,12 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
                         goto done;
                     }
                     /* Decompressed block. Add it to the file data */
-                    decompressed_tmp = cli_realloc(decompressed, (size_t)decompressed_size + decompressed_block_size);
-                    if (NULL == decompressed_tmp) {
-                        cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
-                                   decompressed_size);
-                        free(decompressed_block);
-                        status = CL_EMEM;
-                        goto done;
-                    }
-                    decompressed = decompressed_tmp;
+                    CLI_REALLOC(decompressed,
+                                (size_t)decompressed_size + decompressed_block_size,
+                                cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
+                                           decompressed_size),
+                                free(decompressed_block),
+                                status = CL_EMEM);
 
                     memcpy(decompressed + decompressed_size, decompressed_block, decompressed_block_size);
                     decompressed_size += decompressed_block_size;
@@ -2440,15 +2415,12 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
                         goto done;
                     }
                     /* Decompressed block. Add it to the file data */
-                    decompressed_tmp = cli_realloc(decompressed, (size_t)decompressed_size + decompressed_block_size);
-                    if (NULL == decompressed_tmp) {
-                        cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
-                                   decompressed_size);
-                        free(decompressed_block);
-                        status = CL_EMEM;
-                        goto done;
-                    }
-                    decompressed = decompressed_tmp;
+                    CLI_REALLOC(decompressed,
+                                (size_t)decompressed_size + decompressed_block_size,
+                                cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
+                                           decompressed_size),
+                                free(decompressed_block),
+                                status = CL_EMEM);
 
                     memcpy(decompressed + decompressed_size, decompressed_block, decompressed_block_size);
                     decompressed_size += decompressed_block_size;
@@ -2482,15 +2454,12 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
                     //     goto done;
                     // }
                     // /* Decompressed block. Add it to the file data */
-                    // decompressed_tmp = cli_realloc(decompressed, (size_t)decompressed_size + decompressed_block_size);
-                    // if (NULL == decompressed_tmp) {
-                    //     cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
-                    //                decompressed_size);
-                    //     free(decompressed_block);
-                    //     status = CL_EMEM;
-                    //     goto done;
-                    // }
-                    // decompressed = decompressed_tmp;
+                    // CLI_REALLOC(decompressed,
+                    //             (size_t)decompressed_size + decompressed_block_size,
+                    //             cli_errmsg("cli_egg_extract_file: Failed to allocate %" PRIu64 " bytes for decompressed file!\n",
+                    //                        decompressed_size),
+                    //             free(decompressed_block),
+                    //             status = CL_EMEM);
 
                     // memcpy(decompressed + decompressed_size, decompressed_block, decompressed_block_size);
                     // decompressed_size += decompressed_block_size;
@@ -2522,21 +2491,22 @@ cl_error_t cli_egg_extract_file(void* hArchive, const char** filename, const cha
     }
 
     cli_dbgmsg("cli_egg_extract_file: File extracted: %s\n", currFile->filename.name_utf8);
-    *filename             = strdup(currFile->filename.name_utf8);
-    *output_buffer        = decompressed;
+    *filename = strdup(currFile->filename.name_utf8);
+
+    *output_buffer = decompressed;
+    decompressed   = NULL;
+
     *output_buffer_length = decompressed_size;
-    status                = CL_SUCCESS;
+
+    status = CL_SUCCESS;
 
 done:
     if (NULL != handle) {
         handle->fileExtractionIndex += 1;
     }
 
-    if (CL_SUCCESS != status) {
-        /* Free buffer */
-        if (NULL != decompressed) {
-            free(decompressed);
-        }
+    if (NULL != decompressed) {
+        free(decompressed);
     }
 
     return status;
