@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2022 Cisco and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2014-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Author: Shawn Webb
  *
@@ -229,14 +229,79 @@ int is_object_reference(char *begin, char **endchar, uint32_t *id)
 static char *pdf_decrypt_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *in, size_t *length)
 {
     enum enc_method enc;
+    const char *hex   = NULL;
+    const char *bin   = NULL;
+    char *decoded_bin = NULL;
+    char *dec         = NULL;
+    size_t bin_length;
 
     /* handled only once in cli_pdf() */
     // pdf_handle_enc(pdf);
     if (pdf->flags & (1 << DECRYPTABLE_PDF)) {
+        int hex2str_ret;
+        bool hex_encoded_binary = false;
+        const char *start       = NULL;
+        const char *end         = NULL;
+
         enc = get_enc_method(pdf, obj);
-        return decrypt_any(pdf, obj->id, in, length, enc);
+
+        if (*length < 2) {
+            cli_dbgmsg("pdf_decrypt_string: length < 2\n");
+            return NULL;
+        }
+
+        // Strip off the leading `<` and trailing `>`
+        start = in;
+        if (start[0] == '<') {
+            start++;
+            hex_encoded_binary = true;
+        }
+        end = in + *length;
+        if (end[-1] == '>') {
+            end--;
+        }
+
+        *length = (end - start);
+
+        if (hex_encoded_binary) {
+            hex        = start;
+            bin_length = *length / 2;
+
+            // Convert the hex string to binary
+            decoded_bin = cli_calloc(1, bin_length);
+            if (!decoded_bin) {
+                return NULL;
+            }
+
+            hex2str_ret = cli_hex2str_to(hex, decoded_bin, *length);
+            if (hex2str_ret != 0) {
+                cli_dbgmsg("pdf_decrypt_string: cli_hex2str_to() failed\n");
+                goto done;
+            }
+
+            bin = decoded_bin;
+        } else {
+            // Binary is just embedded directly in the file, no encoding.
+            bin        = start;
+            bin_length = *length;
+        }
+
+        // Decrypt the binary
+        dec = decrypt_any(pdf, obj->id, bin, &bin_length, enc);
+        if (!dec) {
+            cli_dbgmsg("pdf_decrypt_string: decrypt_any() failed\n");
+            goto done;
+        }
+
+        *length = bin_length;
     }
-    return NULL;
+
+done:
+    if (NULL != decoded_bin) {
+        free(decoded_bin);
+    }
+
+    return dec;
 }
 
 char *pdf_finalize_string(struct pdf_struct *pdf, struct pdf_obj *obj, const char *in, size_t len)
