@@ -1,7 +1,7 @@
 /*
  *  Extract VBA source code for component MS Office Documents
  *
- *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog, Nigel Horne
@@ -358,7 +358,7 @@ static size_t vba_normalize(unsigned char *buffer, size_t size)
  * Read a VBA project in an OLE directory.
  * Contrary to cli_vba_readdir, this function uses the dir file to locate VBA modules.
  */
-cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, const char *hash, uint32_t which, int *tempfd, int *has_macros)
+cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, const char *hash, uint32_t which, int *tempfd, int *has_macros, char **tempfile)
 {
     cl_error_t ret = CL_SUCCESS;
     char fullname[1024];
@@ -367,7 +367,6 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
     size_t data_len;
     size_t data_offset;
     const char *stream_name = NULL;
-    char *tempfile          = NULL;
     uint16_t codepage       = CODEPAGE_ISO8859_1;
     unsigned i;
     char *mbcs_name = NULL, *utf16_name = NULL;
@@ -375,7 +374,7 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
     unsigned char *module_data = NULL, *module_data_utf8 = NULL;
     size_t module_data_size = 0, module_data_utf8_size = 0;
 
-    if (dir == NULL || hash == NULL || tempfd == NULL || has_macros == NULL) {
+    if (dir == NULL || hash == NULL || tempfd == NULL || has_macros == NULL || tempfile == NULL) {
         return CL_EARG;
     }
 
@@ -398,12 +397,12 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
 
     *has_macros = *has_macros + 1;
 
-    if ((ret = cli_gentempfd_with_prefix(ctx->sub_tmpdir, "vba_project", &tempfile, tempfd)) != CL_SUCCESS) {
+    if ((ret = cli_gentempfd_with_prefix(ctx->sub_tmpdir, "vba_project", tempfile, tempfd)) != CL_SUCCESS) {
         cli_warnmsg("vba_readdir_new: VBA project cannot be dumped to file\n");
         goto done;
     }
 
-    cli_dbgmsg("Dumping VBA project from dir %s to file %s\n", fullname, tempfile);
+    cli_dbgmsg("Dumping VBA project from dir %s to file %s\n", fullname, *tempfile);
 
 #define CLI_WRITEN(msg, size)                                                 \
     do {                                                                      \
@@ -463,15 +462,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
     CLI_WRITEN("REM VBA project extracted from Microsoft Office document\n\n", 58);
 
     for (data_offset = 0; data_offset < data_len;) {
-        uint16_t id;
-        uint32_t size;
+        uint16_t id, val16;
+        uint32_t size, val32;
 
         if (sizeof(uint16_t) > data_len - data_offset) {
             cli_warnmsg("vba_readdir_new: Failed to read record type from dir\n");
             ret = CL_EREAD;
             goto done;
         }
-        id = le16_to_host(*(uint16_t *)&data[data_offset]);
+        memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+        id = le16_to_host(val16);
         data_offset += sizeof(uint16_t);
 
         if (sizeof(uint32_t) > data_len - data_offset) {
@@ -479,7 +479,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
             ret = CL_EREAD;
             goto done;
         }
-        size = le32_to_host(*(uint32_t *)&data[data_offset]);
+        memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+        size = le32_to_host(val32);
         data_offset += sizeof(uint32_t);
 
         if (size > data_len - data_offset) {
@@ -496,7 +497,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint32_t sys_kind = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t sys_kind = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
                 CLI_WRITEN("REM PROJECTSYSKIND: ", 20);
                 switch (sys_kind) {
@@ -532,7 +534,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint32_t lcid = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t lcid = le32_to_host(val32);
                 char buf[64];
                 data_offset += size;
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTLCID: 0x%08x\n", lcid);
@@ -549,7 +552,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint32_t lcid_invoke = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t lcid_invoke = le32_to_host(val32);
                 char buf[64];
                 data_offset += sizeof(uint32_t);
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTLCIDINVOKE: 0x%08x\n", lcid_invoke);
@@ -566,7 +570,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                codepage = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                codepage = le16_to_host(val16);
                 char buf[64];
                 data_offset += sizeof(uint16_t);
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTCODEPAGE: 0x%04x\n", codepage);
@@ -635,7 +640,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                id = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                id = le16_to_host(val16);
                 if (id != 0x003d) {
                     cli_warnmsg("vba_readdir_new: PROJECTHELPFILEPATH is not followed by PROJECTHELPFILEPATH2\n");
                     CLI_WRITEN("REM WARNING: PROJECTHELPFILEPATH is not followed by PROJECTHELPFILEPATH2\n", 73);
@@ -649,7 +655,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
                 uint32_t size2;
-                size2 = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size2 = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
 
                 if (size2 > data_len - data_offset) {
@@ -685,7 +692,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint32_t context = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t context = le32_to_host(val32);
                 char buf[64];
                 data_offset += size;
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTHELPCONTEXT: 0x%04x\n", context);
@@ -702,7 +710,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint32_t libflags = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t libflags = le32_to_host(val32);
                 char buf[64];
                 data_offset += sizeof(uint32_t);
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTLIBFLAGS: 0x%04x\n", libflags);
@@ -720,7 +729,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint32_t major = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t major = le32_to_host(val32);
                 data_offset += size;
 
                 if (sizeof(uint16_t) > data_len - data_offset) {
@@ -728,7 +738,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint16_t minor = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                uint16_t minor = le16_to_host(val16);
                 data_offset += sizeof(uint16_t);
                 char buf[64];
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTVERSION: %u.%u\n", major, minor);
@@ -745,7 +756,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint16_t modules = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                uint16_t modules = le16_to_host(val16);
                 data_offset += sizeof(uint16_t);
                 char buf[64];
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTMODULES: %u\n", modules);
@@ -762,7 +774,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     ret = CL_EREAD;
                     goto done;
                 }
-                uint16_t cookie = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                uint16_t cookie = le16_to_host(val16);
                 data_offset += sizeof(uint16_t);
                 char buf[64];
                 int buf_length       = snprintf(buf, sizeof(buf), "REM PROJECTCOOKIE: 0x%04x\n", cookie);
@@ -795,14 +808,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x0047) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x0047) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULENAMEUNICODE (0x47) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
                 CLI_WRITEN("\nREM MODULENAMEUNICODE: ", 24);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
 
                 if (size > data_len - data_offset) {
@@ -844,14 +859,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x001a) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x001a) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULESTREAMNAME (0x1a) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
                 CLI_WRITEN("\nREM MODULESTREAMNAME: ", 23);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
 
                 if (size > data_len - data_offset) {
@@ -877,14 +894,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x0032) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x0032) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULESTREAMNAMEUNICODE (0x32) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
                 CLI_WRITEN("\nREM MODULESTREAMNAMEUNICODE: ", 30);
-                uint32_t module_stream_name_size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t module_stream_name_size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
 
                 if (module_stream_name_size > data_len - data_offset) {
@@ -927,14 +946,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x001c) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x001c) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULEDOCSTRING (0x1c) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
                 CLI_WRITEN("\nREM MODULEDOCSTRING: ", 22);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
 
                 if (size > data_len - data_offset) {
@@ -960,14 +981,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x0048) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x0048) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULEDOCSTRINGUNICODE (0x32) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
                 CLI_WRITEN("\nREM MODULEDOCSTRINGUNICODE: ", 29);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
 
                 if (size > data_len - data_offset) {
@@ -1009,13 +1032,15 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x0031) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x0031) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULEOFFSET (0x31) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
                 if (size != sizeof(uint32_t)) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULEOFFSET record size");
@@ -1029,7 +1054,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                uint32_t module_offset = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t module_offset = le32_to_host(val32);
                 data_offset += size;
                 char buffer[64];
                 int buffer_size = snprintf(buffer, sizeof(buffer), "\nREM MODULEOFFSET: 0x%08x", module_offset);
@@ -1045,14 +1071,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x001e) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x001e) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULEHELPCONTEXT (0x1e) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
 
                 data_offset += sizeof(uint16_t);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
                 if (size != sizeof(uint32_t)) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULEHELPCONTEXT record size");
@@ -1066,7 +1094,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                uint32_t help_context = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                uint32_t help_context = le32_to_host(val32);
                 data_offset += size;
                 buffer_size = snprintf(buffer, sizeof(buffer), "\nREM MODULEHELPCONTEXT: 0x%08x", help_context);
                 if (buffer_size > 0) {
@@ -1081,13 +1110,15 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                if ((id = le16_to_host(*(uint16_t *)&data[data_offset])) != 0x002c) {
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                if ((id = le16_to_host(val16)) != 0x002c) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULECOOKIE (0x2c) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
                 if (size != sizeof(uint16_t)) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULECOOKIE record size");
@@ -1101,7 +1132,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                uint16_t cookie = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                uint16_t cookie = le16_to_host(val16);
                 data_offset += size;
                 buffer_size = snprintf(buffer, sizeof(buffer), "\nREM MODULECOOKIE: 0x%04x", cookie);
                 if (buffer_size > 0) {
@@ -1115,14 +1147,16 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                id = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                id = le16_to_host(val16);
                 if (id != 0x0021 && id != 0x0022) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULETYPE (0x21/0x22) record, but got 0x%04x\n", id);
                     ret = CL_EREAD;
                     goto done;
                 }
                 data_offset += sizeof(uint16_t);
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
                 if (size != 0) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULETYPE record size");
@@ -1142,7 +1176,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                id = le16_to_host(*(uint16_t *)&data[data_offset]);
+                memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                id = le16_to_host(val16);
                 data_offset += sizeof(uint16_t);
 
                 if (id == 0x0025) {
@@ -1152,7 +1187,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                         goto done;
                     }
 
-                    size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                    memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                    size = le32_to_host(val32);
                     data_offset += sizeof(uint32_t);
                     if (size != 0) {
                         cli_dbgmsg("cli_vba_readdir_new: Expected MODULEREADONLY record size");
@@ -1167,7 +1203,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                         goto done;
                     }
 
-                    id = le16_to_host(*(uint16_t *)&data[data_offset]);
+                    memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                    id = le16_to_host(val16);
                     data_offset += sizeof(uint16_t);
                 }
 
@@ -1179,7 +1216,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                         goto done;
                     }
 
-                    size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                    memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                    size = le32_to_host(val32);
                     data_offset += sizeof(uint32_t);
                     if (size != 0) {
                         cli_dbgmsg("cli_vba_readdir_new: Expected MODULEPRIVATE record size");
@@ -1194,7 +1232,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                         goto done;
                     }
 
-                    id = le16_to_host(*(uint16_t *)&data[data_offset]);
+                    memcpy(&val16, &data[data_offset], sizeof (uint16_t));
+                    id = le16_to_host(val16);
                     data_offset += sizeof(uint16_t);
                 }
 
@@ -1211,7 +1250,8 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                     goto done;
                 }
 
-                size = le32_to_host(*(uint32_t *)&data[data_offset]);
+                memcpy(&val32, &data[data_offset], sizeof (uint32_t));
+                size = le32_to_host(val32);
                 data_offset += sizeof(uint32_t);
                 if (size != 0) {
                     cli_dbgmsg("cli_vba_readdir_new: Expected MODULETERMINATOR record size");
@@ -1259,10 +1299,20 @@ cl_error_t cli_vba_readdir_new(cli_ctx *ctx, const char *dir, struct uniq *U, co
                         module_data_utf8_size = vba_normalize(module_data_utf8, module_data_utf8_size);
 
                         CLI_WRITEN(module_data_utf8, module_data_utf8_size);
+
+                        if (NULL != ctx->engine->cb_vba) {
+                            ctx->engine->cb_vba(module_data_utf8, module_data_utf8_size, ctx->cb_ctx);
+                        }
+
                         module_stream_found = 1;
                         free(module_data_utf8);
                         module_data_utf8 = NULL;
                     } else {
+                        /*If normalization didn't work, fall back to the pre-normalized data.*/
+                        if (NULL != ctx->engine->cb_vba) {
+                            ctx->engine->cb_vba(module_data, module_data_size, ctx->cb_ctx);
+                        }
+
                         CLI_WRITEN("\n<Error decoding module data>\n", 30);
                         cli_dbgmsg("cli_vba_readdir_new: Failed to decode VBA module content from codepage %" PRIu16 " to UTF8\n", codepage);
                     }
@@ -1305,9 +1355,6 @@ done:
     if (stream_name) {
         free((void *)stream_name);
     }
-    if (tempfile) {
-        free(tempfile);
-    }
     if (ret != CL_SUCCESS && *tempfd >= 0) {
         close(*tempfd);
         *tempfd = -1;
@@ -1799,6 +1846,8 @@ ppt_unlzw(const char *dir, int fd, uint32_t length)
         return FALSE;
     }
 
+    memset(&stream, 0, sizeof(stream));
+
     stream.zalloc    = Z_NULL;
     stream.zfree     = Z_NULL;
     stream.opaque    = (void *)NULL;
