@@ -14,20 +14,16 @@ CmdExtract::CmdExtract(CommandData *Cmd)
   TotalFileCount=0;
 
   // Common for all archives involved. Set here instead of DoExtract()
-  // to use in unrar.dll too. Allows to avoid LinksToDirs() calls
-  // and save CPU time in no symlinks including ".." in target were extracted.
-#if defined(_WIN_ALL)
-  // We can't expand symlink path components in another symlink target
-  // in Windows. We can't create symlinks in Android now. Even though we do not
-  // really need LinksToDirs() calls in these systems, we still call it
-  // for extra safety, but only if symlink with ".." in target was extracted.
-  ConvertSymlinkPaths=false;
-#else
+  // to use in unrar.dll too.
   // We enable it by default in Unix to care about the case when several
   // archives are unpacked to same directory with several independent RAR runs.
   // Worst case performance penalty for a lot of small files seems to be ~3%.
+  // 2023.09.15: Windows performance impact seems to be negligible,
+  // less than 0.5% when extracting mix of small files and folders.
+  // So for extra security we enabled it for Windows too, even though
+  // unlike Unix, Windows doesn't expand lnk1 in symlink targets like
+  // "lnk1/../dir", but converts such path to "dir".
   ConvertSymlinkPaths=true;
-#endif
 
   Unp=new Unpack(&DataIO);
 #ifdef RAR_SMP
@@ -1463,7 +1459,7 @@ bool CmdExtract::CheckUnpVer(Archive &Arc,const wchar *ArcFileName)
 //
 // Alternatively we could collect references while extracting an archive
 // and perform the second extraction pass for references only.
-// But it would be slower for solid archives than scaning headers
+// But it would be slower for solid archives than scanning headers
 // in first pass and extracting everything in second, as implemented now.
 //
 void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumbering)
